@@ -25,9 +25,24 @@ def _isSBMLModel(obj):
     return False
 
 def _checkSBMLDocument(document): 
+  document.getErrorLog().getNumFailsWithSeverity(2)
   if (document.getNumErrors() > 0):
     raise ValueError("Errors in SBML document")
 
+
+# Some convenience methods    
+def loadSBMLStr (sbmlStr):
+    """
+    Load an SBML model in the form of a string and return an instance to SBMLModel
+    """
+    return SbmlModel (sbmlStr=sbmlStr)
+
+def loadSBMLFile (sbmlFile):
+    """
+    Load an SBML model from a file and return an instance to SBMLModel
+    """
+    return SbmlModel (sbmlFile=sbmlFile)
+
 class SbmlModel(object):
 
     """   
@@ -97,7 +112,7 @@ class SbmlModel(object):
 
             _checkSBMLDocument(self.document)
             self.model = self.document.getModel()
-             
+    
     def _check(self, value, message):
         if value == None:
             raise SystemExit('LibSBML returned a null value trying to ' + \
@@ -481,7 +496,7 @@ class SbmlModel(object):
     #def getSpecies(self, species_id):      
     #    return self.model.getSpecies(species_id)
 
-    def getListOfCompartments(self):
+    def getListOfCompartmentIds(self):
         """
         Returns a list of compartment Ids
         """
@@ -492,6 +507,13 @@ class SbmlModel(object):
            alist.append (comp.getId())
         return alist
     
+
+    def getCompartmentId (self, index):
+        """
+        Returns the indexth compartment from the list of compartments
+        """
+        return self.getListOfCompartmentIds()[index]
+
     def getCompartmentVolume (self, Id):
         """
         **Parameters**
@@ -521,13 +543,13 @@ class SbmlModel(object):
     
     def getNthFloatingSpeciesId (self, index):
         """
-        Returns the Id of the nth floating speces
+        Returns the Id of the nth floating species
         """
         return self.getListOfFloatingSpecies()[index]
 
     def getNthBoundarySpeciesId (self, index):
         """
-        Returns the Id of the nth boundary speces
+        Returns the Id of the nth boundary species
         """
         return self.getListOfBoundarySpecies()[index]
 
@@ -618,7 +640,7 @@ class SbmlModel(object):
         """
         sp = self.model.getSpecies(Id)
         if sp != None:
-           if sp.isSetBoundaryCondition():
+           if sp.getBoundaryCondition():
               return False
            else:
               return True
@@ -631,7 +653,7 @@ class SbmlModel(object):
         """
         sp = self.model.getSpecies(Id)
         if sp != None:
-           if sp.isSetBoundaryCondition():
+           if sp.getBoundaryCondition():
               return True
            else:
               return False
@@ -657,7 +679,7 @@ class SbmlModel(object):
         """
         sp = self.model.getSpecies(Id)
         if sp != None:
-            if sp.isSetInitialConcentration():
+            if sp.getInitialConcentration():
                 return True
             else:
                 return False
@@ -677,10 +699,7 @@ class SbmlModel(object):
         """
         return len (self.getListOfBoundarySpecies())
     
-    #def getParameter(self, param_id):
-    #    return self.model.getParameter(param_id)
-
-    def getListOfParameters(self):
+    def getListOfParameterIds(self):
         """
         Returns a list of all global parameter Ids in the model.
         """
@@ -691,6 +710,11 @@ class SbmlModel(object):
             alist.append (p.getId())             
         return alist
 
+    def getParameterId (self, index):
+        """
+        Returns the indexth parameter from the list of parameter
+        """
+        return self.getListOfParameterIds()[index]
     
     def isParameterValueSet (self, Id):
         """
@@ -718,11 +742,15 @@ class SbmlModel(object):
         Example: value = model.getParameterValue ('k1')       
         """      
         p = self.model.getParameter(Id)
-        if p.isSetValue():
-           return p.getValue()
-        raise Exception ('Parameter does not exist')      
+        if p != None:
+           if p.isSetValue():
+              return p.getValue()
+           else:
+              return 0
+        else:
+           raise Exception ('Parameter does not exist')      
         
-    def getListOfReactions(self):
+    def getListOfReactionIds (self):
         """
         Returns a list of all reaction Ids.
         """       
@@ -733,6 +761,12 @@ class SbmlModel(object):
             alist.append (p.getId())             
         return alist
     
+    def getNthReactionId (self, index):
+        """
+        Returns the Id of the nth reaction
+        """
+        return self.getListOfReactionIds()[index]
+
     def getNumReactants (self, Id):
         """
          Returns the number of reactants in the reaction given by the Id argument.
@@ -827,8 +861,11 @@ class SbmlModel(object):
         Example: stInt = model.getReactantStoichiometry ('J1', 0)
         """         
         ra = self.model.getReaction(reactionId)
-        sr = ra.getReactant(reactantIndex)
-        return sr.getStoichiometry ()
+        if ra != None:
+           sr = ra.getReactant(reactantIndex)
+           return sr.getStoichiometry ()
+        else:
+           raise Exception ('Reaction:' + reactionId + ' does not exist') 
 
     def getProductStoichiometry (self, reactionId, productIndex):
         """
@@ -847,8 +884,11 @@ class SbmlModel(object):
         Example: stInt = model.getProductStoichiometry ('J1', 0)
         """
         ra = self.model.getReaction(reactionId)
-        sr = ra.getProduct(productIndex)
-        return sr.getStoichiometry ()
+        if ra != None:
+           sr = ra.getProduct(productIndex)
+           return sr.getStoichiometry ()
+        else:
+           raise Exception ('Reaction:' + reactionId + ' does not exist') 
         
     def getNumModifiers (self, reactionId):
         """
@@ -864,12 +904,12 @@ class SbmlModel(object):
         reaction = self.model.getReaction (reactionId)
         if reaction != None:
            for i in range (reaction.getNumModifiers()):
-               alist.append (reaction.getModifer (0))
+               alist.append (reaction.getModifier (0).getSpecies())
            return alist
         else:
            raise Exception ('No such reaction can be found:', reactionId)
  
-    def getListOfRules(self):
+    def getListOfRuleIds(self):
         """
         Returns a list of Ids for the rules
         """ 
@@ -886,7 +926,9 @@ class SbmlModel(object):
 
         Example: rule = model.getRuleId (2)
         """         
-        return self.model.getRule (index).getId()
+        rule = self.model.getRule (index)
+        if rule != None:
+            return rule.getId()
 
     def getRuleRightSide (self, rule):
         """
@@ -896,7 +938,9 @@ class SbmlModel(object):
 
         Example: formula = model.getRuleRightSide (0)
         """
-        return self.model.getRule (rule).getFormula()
+        rule = self.model.getRule (rule)
+        if rule != None:
+            return rule.getFormula()
 
     def getRuleType (self, rule):
         """
@@ -907,15 +951,60 @@ class SbmlModel(object):
         Example: ruleStr = model.getRuleType (0)
         """
         myRule = self.model.getRule (rule)
-        t1 = myRule.getTypeCode()
-        if t1 == libsbml.SBML_RATE_RULE:
-           return 'ODE (or rate) rule' 
-        if t1 == libsbml.SBML_ASSIGNMENT_RULE:
-           return 'Assignment rule' 
-        if t1 == libsbml.SBML_ALGEBRAIC_RULE:
-           return 'Algebraic rule' 
-        raise Exception ('Unknown rule in SBML model')
+        if myRule != None:
+           t1 = myRule.getTypeCode()
+           if t1 == libsbml.SBML_RATE_RULE:
+              return 'ODE (or rate) rule' 
+           if t1 == libsbml.SBML_ASSIGNMENT_RULE:
+              return 'Assignment rule' 
+           if t1 == libsbml.SBML_ALGEBRAIC_RULE:
+              return 'Algebraic rule' 
+           raise Exception ('Unknown rule in SBML model')
 
+    def isRuleType_Assignment (self, rule):
+        """
+        Returns true the rule is an assignemnt type
+        """
+        myRule = self.model.getRule (rule)
+        if myRule != None:
+           t1 = myRule.getTypeCode()
+           if t1 == libsbml.SBML_ASSIGNMENT_RULE:
+              return True
+           else:
+              return False 
+        else:
+           return None
+           #raise Exception ('The rule: ' + str (rule) + ', does not exist') 
+
+    def isRuleType_Rate (self, rule):
+        """
+        Returns true the rule is an rate rule (ode) type
+        """
+        myRule = self.model.getRule (rule)
+        if myRule != None:
+           t1 = myRule.getTypeCode()
+           if t1 == libsbml.SBML_RATE_RULE:
+              return True
+           else:
+              return False 
+        else:
+           raise Exception ('The rule: ' + str (rule) + ', does not exist') 
+
+    def isRuleType_Algebraic (self, rule):
+        """
+        Returns true the rule is an algebraic rule type
+        """
+        myRule = self.model.getRule (rule)
+        if myRule != None:
+           t1 = myRule.getTypeCode()
+           if t1 == libsbml.SBML_ALGEBRAIC_RULE:
+              return True
+           else:
+              return False 
+        else:
+           raise Exception ('The rule: ' + str (rule) + ', does not exist') 
+
+    #  ----------------------- EVENTS -------------------------
     def getEventId (self, index):
         """
         Returns the Id for the indexth event
@@ -1046,38 +1135,17 @@ class SbmlModel(object):
             alist.append (libsbml.formulaToL3String(myFunc.getArgument(i)))
         return alist
 
-    # def getReaction(self, rxn_id):
-    #     return self.model.getReaction(rxn_id)
-
-    # def getCompartment(self, comp_id):
-    #     return self.model.getCompartment(comp_id)
-
-    # def getListOfEvents(self):
-    #     return self.model.getListOfEvents()
-
-    # def getEvent(self, event_id):
-    #     return self.model.getEvent(event_id)
-
-    # def getRule(self, var):
-    #     return self.model.getRule(var)
-
-    # def getInitialAssignment(self, var):
-    #     return self.model.getInitialAssignment(var)
-
-    # def getListOfInitialAssignments(self):
-    #     return self.model.getListOfInitialAssignments()
-
     def toSBML(self):
         """Returns the model in SBML format as a string.  Also checks model consistency
-       and prints all errors and warnings.
-
-       Example: print (model.toSBML())
-       """
+        and prints all errors and warnings.
 
+        Example: print (model.toSBML())
+        """
+        self.document.setConsistencyChecks(libsbml.LIBSBML_CAT_UNITS_CONSISTENCY, False)
         errors = self.document.checkConsistency()
         if (errors > 0):
-            for i in range(errors):
-                print(self.document.getError(i).getSeverityAsString(), ": ", self.document.getError(i).getMessage())
+           for i in range(errors):
+               print(self.document.getError(i).getSeverityAsString(), ": ", self.document.getError(i).getMessage()) 
 
         return libsbml.writeSBMLToString(self.document)
 
